/*
 *  Copyright (C) 2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "converterFactory.h"

#include "converters.h"

cFactory* cFactory::instance() {

	static cFactory o;
	return &o;
}

converter* cFactory::get(const unsigned int srIn, const unsigned int srOut,
		const size_t size, const unsigned int channels,
		const sample_t inputPrecision, const sample_t outputPrecision,
		const unsigned int fract) {

	switch (inputPrecision) {
	case U8:
		return (srIn!=srOut)?new resampler<FXuchar, FXuchar>(srIn, srOut, size, channels, new quantizerVoid<FXuchar>()):0;
	case S16:
		return (srIn!=srOut)?new resampler<FXshort, FXshort>(srIn, srOut, size, channels, new quantizerVoid<FXshort>()):0;
	case S24:
	case S32:
		return 0;
	case SAMPLE_FLOAT:
		if (srIn!=srOut) {
			if (outputPrecision==U8) {
				FXTRACE((10, "resampler float->U8\n"));
				return (converter*)new resampler<FXfloat, FXuchar>(srIn, srOut, size, channels, new quantizerFloat<FXuchar>());
			} else {
				FXTRACE((10, "resampler float->S16\n"));
				return (converter*)new resampler<FXfloat, FXshort>(srIn, srOut, size, channels, new quantizerFloat<FXshort>());
			}
		} else {
			if (outputPrecision==U8) {
				FXTRACE((10, "converter float->U8\n"));
				return (converter*)new converterDecimal<FXfloat, FXuchar>(size, channels, new quantizerFloat<FXuchar>());
			} else {
				FXTRACE((10, "converter float->S16\n"));
				return (converter*)new converterDecimal<FXfloat, FXshort>(size, channels, new quantizerFloat<FXshort>());
			}
		}
	case SAMPLE_FIXED:
		if (srIn!=srOut) {
			if (outputPrecision==U8) {
				FXTRACE((10, "resampler fixed->U8\n"));
				return (converter*)new resampler<FXint, FXuchar>(srIn, srOut, size, channels, new quantizerFixed<FXuchar>(fract));
			} else {
				FXTRACE((10, "resampler fixed->S16\n"));
				return (converter*)new resampler<FXint, FXshort>(srIn, srOut, size, channels, new quantizerFixed<FXshort>(fract));
			}
		} else {
			if (outputPrecision==U8) {
				FXTRACE((10, "converter fixed->U8\n"));
				return (converter*)new converterDecimal<FXint, FXuchar>(size, channels, new quantizerFixed<FXuchar>(fract));
			} else {
				FXTRACE((10, "converter fixed->S16\n"));
				return (converter*)new converterDecimal<FXint, FXshort>(size, channels, new quantizerFixed<FXshort>(fract));
			}
		}
	default:
		return 0;
	}
}
