/*
 *  Copyright (C) 2009-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "converters.h"

template <typename I, typename O>
size_t resampler<I, O>::convert(const void* buf, const size_t len) {

	I* const in=(I*)_buffer;
	O* const out=(O*)buf;

	unsigned int idx=0;
	FXuint error=0;
	const size_t m=len/sizeof(O);
	for (size_t j=0; j<m; j+=_channels) {
		for (unsigned int c=0; c<_channels; c++) {
			const I val=in[idx+c];
			out[j+c]=_quantizer->get(val+(FXshort)(((FXuint)(in[idx+c+_channels]-val)*error)>>16), c);
		}
		error+=_rate;
		while (error>=0x10000) {
			error-=0x10000;
			idx+=_channels;
		}
	}

	const size_t l=_bufsize/sizeof(I);
	FXTRACE((10, "resamplerDecimal idx: %d; l: %d\n", idx, l));

	_data=(l<idx)?(l-idx)*sizeof(I):0;
	for (size_t j=idx; j<l; j+=_channels) {
		for (unsigned int c=0; c<_channels; c++)
			in[c]=in[j+c];
	}

	return len;
}

template size_t resampler<FXuchar, FXuchar>::convert(const void* buf, const size_t len);
template size_t resampler<FXshort, FXshort>::convert(const void* buf, const size_t len);
template size_t resampler<FXint, FXuchar>::convert(const void* buf, const size_t len);
template size_t resampler<FXint, FXshort>::convert(const void* buf, const size_t len);
template size_t resampler<FXfloat, FXuchar>::convert(const void* buf, const size_t len);
template size_t resampler<FXfloat, FXshort>::convert(const void* buf, const size_t len);

/******************************************************************************/

template <typename I, typename O>
size_t converterDecimal<I, O>::convert(const void* buf, const size_t len) {

	I* const in=(I*)_buffer;
	O* const out=(O*)buf;

	const size_t m=len/sizeof(O);
	for (size_t j=0; j<m; j+=_channels) {
		for (unsigned int c=0; c<_channels; c++) {
			out[j+c]=_quantizer->get(in[j+c], c);
		}
	}

	return len;
}

template size_t converterDecimal<FXint, FXuchar>::convert(const void* buf, const size_t len);
template size_t converterDecimal<FXint, FXshort>::convert(const void* buf, const size_t len);
template size_t converterDecimal<FXfloat, FXuchar>::convert(const void* buf, const size_t len);
template size_t converterDecimal<FXfloat, FXshort>::convert(const void* buf, const size_t len);
