/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef QUANTIZER_H
#define QUANTIZER_H

#include <fx.h>

template <typename I, typename O>
class quantizer {

public:
	/// Get dithered sample
	virtual O get(const I sample, const unsigned int channel) =0;
};

/******************************************************************************/

template <typename T>
class quantizerVoid : public quantizer<T, T> {

private:
	quantizerVoid(const quantizerVoid&);
	quantizerVoid& operator=(const quantizerVoid&);
public:
	quantizerVoid() { FXTRACE((10, "quantizerVoid %d bytes\n", sizeof(T))); };
	~quantizerVoid() {};

	/// Get dithered sample
	inline T get(const T sample, const unsigned int channel) { return sample; }
};

/******************************************************************************/

template<typename O>
class quantizerFixed : public quantizer<FXint, O> {

protected:
	FXuint	_random[2][2];

	FXint	_scalebits;
	FXint	_clip;
	FXint	_mask;

private:
	quantizerFixed(const quantizerFixed&);
	quantizerFixed& operator=(const quantizerFixed&);

	FXint get32(const FXint sample, const unsigned int channel);

public:
	quantizerFixed(const unsigned int fract);
	~quantizerFixed() {};

	/// Get dithered sample
	O get(const FXint sample, const unsigned int channel);
};

/******************************************************************************/

template<typename O>
class quantizerFloat : public quantizer<FXfloat, O> {

protected:
	FXuint	_random[2][2];

private:
	quantizerFloat(const quantizerFloat<O>&);
	quantizerFloat<O>& operator=(const quantizerFloat<O>&);

	FXint get32(const FXfloat sample, const unsigned int channel, const FXint max);

public:
	quantizerFloat();
	~quantizerFloat() {};

	/// Get dithered sample
	O get(const FXfloat sample, const unsigned int channel);
};

#endif
