/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef GAME_H
#define GAME_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_GME
#include "gme/Music_Emu.h"

#include <fx.h>
#include "inputBackend.h"
#include "stilview/stil.h"

/*****************************************************************/

typedef struct {
	FXint		samplerate;
	FXString	asmaPath;
} gmeConfig_t;

/*****************************************************************/

#include "configFrame.h"

class gmeConfig : public configFrame {
	FXDECLARE(gmeConfig)

private:
	FXDataTarget	_asmaDir;

private:
	gmeConfig() {};
	gmeConfig(const gmeConfig&);
	gmeConfig& operator=(const gmeConfig&);

public:
	gmeConfig(FXPacker* win);
	virtual ~gmeConfig() {};

public:
	enum{
		ID_SAMPLERATE=FXDialogBox::ID_LAST,
		ID_ASMA,
		ID_LAST,
	};

public:
	long onCmdSamplerate(FXObject*, FXSelector, void*);
	long onCmdAsma(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class gmeBackend : public inputBackend {

	friend class gmeConfig;

private:
	static gmeConfig_t	_settings;

private:
	Music_Emu	*_emu;
	bool		_hasStilInfo;

	STIL		*_stil;

private:
	gmeBackend();

	void openAsma(const FXString& asmaPath);

	void getInfo();

	bool check(const char* error);

public:
	~gmeBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new gmeBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get number of subtunes
	unsigned int subtunes() const { return _emu?_emu->track_count():0; }

	/// Get current subtune
	unsigned int subtune() const { return _emu?_emu->current_track()+1:0; }

	/// Change subtune
	bool subtune(const unsigned int i);

	/// Get samplerate
	unsigned int samplerate() const { return _settings.samplerate; }

	/// Get channels
	unsigned int channels() const { return 2; }

	/// Get precision
	sample_t precision() const { return S16; }

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new gmeConfig(win); }

	/// Get Music directory
	const FXString getMusicDir(void) { return _settings.asmaPath; }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
