/*
 *  Copyright (C) 2007-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef HVL_H
#define HVL_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_HVL
extern "C" {
#include "hvl_replay/hvl_replay.h"
}
#include <fx.h>
#include "inputBackend.h"

/*****************************************************************/

typedef struct {
	FXint		samplerate;
} hvlConfig_t;

/*****************************************************************/

#include "configFrame.h"

class hvlConfig : public configFrame {
	FXDECLARE(hvlConfig)

private:
	hvlConfig() {};
	hvlConfig(const hvlConfig&);
	hvlConfig& operator=(const hvlConfig&);

public:
	hvlConfig(FXPacker* win);
	virtual ~hvlConfig() {};

public:
	enum{
		ID_SAMPLERATE=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdSamplerate(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class hvlBackend : public inputBackend {


	friend class hvlConfig;

private:
	static hvlConfig_t	_settings;

private:
	struct hvl_tune *_tune;

	int		_subtune;

	int		_left;
	FXuint		_size;
	char		*_buffer;

private:
	hvlBackend();

public:
	~hvlBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new hvlBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get number of subtunes
	unsigned int subtunes() const { return _tune?_tune->ht_SubsongNr:0; }

	/// Get current subtune
	unsigned int subtune() const { return _tune?_tune->ht_SongNum:0; }

	/// Change subtune
	bool subtune(const unsigned int i);

	/// Get samplerate
	unsigned int samplerate() const { return _settings.samplerate; }

	/// Get channels
	unsigned int channels() const { return 2; }

	/// Get precision
	sample_t precision() const { return S16; }

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new hvlConfig(win); }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
