/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef INPUT_PLUGIN_H
#define INPUT_PLUGIN_H

#include "input.h"

#include "utils.h"

#define SECTION	FXStringFormat("%s Settings", _name).text()

#define gettext(x) x

/*****************************************************************/

class inputBackend : public input {

private:
	const char	*_name;
	FXIcon		*_icon;

	unsigned int	_time;

protected:
	metaDataImpl	_metaData;

private:
	inputBackend();
	inputBackend(const inputBackend&);
	inputBackend& operator=(const inputBackend&);

protected:
	inputBackend(const char name[], const unsigned char iconType[]=0);

	/// Set song duration
	void time(unsigned int newTime) { _time=newTime; }

	/// Load int setting
	FXint load(const FXchar* key, FXint defVal) {
		return FXApp::instance()->reg().readIntEntry(SECTION, key, defVal);
	}

	/// Load string setting
	FXString load(const FXchar* key, FXString defVal) {
		return FXString(FXApp::instance()->reg().readStringEntry(SECTION, key, defVal.text()));
	}

	/// Save int setting
	void save(const FXchar* key, FXint value) {
		FXApp::instance()->reg().writeIntEntry(SECTION, key, value);
	}

	/// Save string setting
	void save(const FXchar* key, FXString value) {
		FXApp::instance()->reg().writeStringEntry(SECTION, key, value.text());
	}

	/// Song is loaded
	void songLoaded(const char* location) { if (location) _metaData.addInfo(metaData::LOCATION, location); else _metaData.clearInfo(); }

	/// Rewind to start
	virtual bool rewind() {}; // called only by seek(), will be removed

public:
	virtual ~inputBackend();

	/// Get backend name
	const char* name() const { return _name; }

	/// Get song info
	virtual const metaData* getMetaData() const { return &_metaData; };

	/// Get song duration
	unsigned int time() const { return _time; }

	/// Get fractional scale for fixed point types
	virtual unsigned int fract() const { return 0; }

	/// Get filetype icon
	FXIcon* icon() const { return _icon; }

	/// Seek specified position
	virtual bool seek(const int pos) { return rewind(); }

	/// Song is loaded
	const char* songLoaded() const { return _metaData.getInfo(metaData::LOCATION); }

	/// Get number of subtunes
	virtual unsigned int subtunes() const { return 0; }

	/// Get current subtune
	virtual unsigned int subtune() const { return 0; }

	/// Change subtune
	virtual bool subtune(const unsigned int i) { return false; }

	/// Gapless support
	virtual bool gapless() const { return false; }

	/// Get Music directory
	virtual const FXString getMusicDir(void) { return FXString::null; }

	virtual void loadSettings() {};

	virtual void saveSettings() {};

};

#endif
