/*
 *  Copyright (C) 2007-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "inputFactory.h"

#include "mpg123Backend.h"
#include "madBackend.h"
#include "oggBackend.h"
#include "sndBackend.h"
#include "ffmpegBackend.h"
#include "modBackend.h"
#include "sidBackend.h"
#include "gmeBackend.h"
#include "hvlBackend.h"
#include "mpcBackend.h"
#include "wvBackend.h"
#include "ttaBackend.h"

iFactory* iFactory::instance() {

	static iFactory i;
	return &i;
}

template <class backend>
void iFactory::regBackend() {

	inputs_t temp;

	temp.name=backend::name;
	temp.supports=backend::supports;
	temp.factory=&backend::factory;
	_inputs.append(temp);
}

iFactory::iFactory() {

	// Register backends

#ifdef HAVE_MPG123_H
	if (mpg123Backend::init())
		regBackend<mpg123Backend>();
#endif

#ifdef HAVE_MAD_H
	if (madBackend::init())
		regBackend<madBackend>();
#endif

#ifdef HAVE_VORBIS
	regBackend<oggBackend>();
#endif

#ifdef HAVE_LIBMODPLUG
	regBackend<modBackend>();
#endif

#ifdef HAVE_SIDPLAY2
	regBackend<sidBackend>();
#endif

#ifdef BUILD_GME
	regBackend<gmeBackend>();
#endif

#ifdef BUILD_HVL
	regBackend<hvlBackend>();
#endif

#ifdef HAVE_LIBMPCDEC
	regBackend<mpcBackend>();
#endif

#ifdef HAVE_WAVPACK
	regBackend<wvBackend>();
#endif

#ifdef BUILD_TTA
	regBackend<ttaBackend>();
#endif

#ifdef HAVE_SNDFILE
	if (sndBackend::init())
		regBackend<sndBackend>();
#endif

#ifdef HAVE_FFMPEG
	if (ffmpegBackend::init())
		regBackend<ffmpegBackend>();
#endif
}

input* iFactory::get(const int i) {

	return _inputs[i].factory();
}
