/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "modBackend.h"

#ifdef HAVE_LIBMODPLUG

#include "utils.h"

// created by reswrap from file modplug.gif
extern const unsigned char iconMod[218]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x10,0x00,0x10,0x00,0xf4,0x13,0x00,0x00,0x00,0x00,
  0x30,0x30,0x30,0x58,0x58,0x58,0x83,0x3b,0x3b,0xb6,0x37,0x37,0xb3,0x3b,0x3b,0x9e,
  0x4f,0x4f,0x92,0x5c,0x5c,0x9b,0x53,0x53,0x8d,0x60,0x60,0x82,0x7d,0x7d,0x8c,0x72,
  0x72,0xa4,0x4a,0x4a,0xa9,0x45,0x45,0xa6,0x59,0x59,0xb4,0x4b,0x4b,0xe9,0x16,0x16,
  0xff,0x00,0x00,0x80,0x80,0x80,0xff,0xff,0xff,0xcc,0xcc,0xcc,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,
  0x01,0x00,0x00,0x14,0x00,0x2c,0x00,0x00,0x00,0x00,0x10,0x00,0x10,0x00,0x00,0x05,
  0x57,0x20,0x25,0x8e,0x64,0x39,0x0a,0x41,0x93,0x48,0x11,0x71,0x30,0xd2,0x80,0x9e,
  0x81,0x64,0x3f,0x76,0x3e,0x8b,0xa8,0xe0,0xff,0xc0,0x00,0x2d,0x40,0x2c,0x16,0x77,
  0x14,0x94,0xa4,0x10,0xc9,0xe9,0x84,0xbc,0x00,0xb3,0x19,0x59,0x28,0x0a,0x12,0x24,
  0x2a,0xc2,0xed,0x76,0xb5,0x52,0x2e,0x13,0x8b,0x05,0xb3,0x9a,0xce,0x2c,0x34,0x19,
  0x00,0xba,0xc1,0xc6,0x38,0xd8,0x91,0x7e,0xd2,0x20,0xb7,0x34,0x38,0x62,0x60,0x21,
  0x24,0x06,0x65,0x6b,0x26,0x84,0x14,0x21,0x00,0x3b
  };

#define EXT	"*.(mod|s3m|xm|it|669|amf|ams|dbm|dmf|dsm|far|mdl|med|mtm|okt|ptm|stm|ult|umx|mt2|psm),(mod).*"

#ifdef HAVE_LIBZ
#  include "unzip/unzip.h"
#ifdef _WIN32
#  include "unzip/iowin32.h"
#endif
#  define EXTZ	",*.(xmz|itz|mdz|s3z)"
#endif

#ifndef _WIN32
#  define UNMO3	"unmo3"
#else
#  define UNMO3	"unmo3.exe"
#endif

#define OVERSAMPLING	0x01
#define NOISE_REDUCTION	0x02

#define CREDITS		"libmodplug"
#define LINK		"http://modplug-xmms.sourceforge.net/"

const char modBackend::name[]="Modplug";

modConfig_t modBackend::_settings;

/*****************************************************************/

size_t modBackend::fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds) {

	return (_module->Read(buffer, bufferSize)*_sampleSize);
}

/*****************************************************************/
#if defined(__GNUC__) && defined(__linux__) && defined(__i386__)
namespace FX {
extern FXAPI FXuint fxcpuid();
}
#endif

bool modBackend::supports(const char* fileName) {

#if FOX_MINOR>6
	if (FXPath::match( FXPath::name(fileName), EXT, NoEscape|PathName|CaseFold))
#else
	if (FXPath::match(FXString(EXT), FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD))
#endif
		return true;
#ifdef HAVE_LIBZ
#if FOX_MINOR>6
	if (FXPath::match(FXPath::name(fileName), EXTZ, NoEscape|PathName|CaseFold))
#else
	if (FXPath::match(FXString(EXTZ), FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD))
#endif
		return true;
#endif
	const FXString unmo3=FXPath::search(FXSystem::getExecPath(), UNMO3);
	if ((!unmo3.empty())
		&& FXStat::isExecutable(unmo3)
#if FOX_MINOR>6
		&& FXPath::match(FXPath::name(fileName), "*.mo3", NoEscape|PathName|CaseFold))
#else
		&& FXPath::match("*.mo3", FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD))
#endif
		return true;
	return false;
}

modBackend::modBackend() :
	inputBackend(name, iconMod),
	_module(0),
#if defined(__GNUC__) && defined(__linux__) && defined(__i386__)
	_mmx(fxcpuid()&0x002)
#else
	_mmx(false)
#endif
 {
	_ext=FXString(EXT);
#ifdef HAVE_LIBZ
	_ext.append(EXTZ);
#endif

	const FXString unmo3=FXPath::search(FXSystem::getExecPath(), UNMO3);

	if ((!unmo3.empty()) && FXStat::isExecutable(unmo3)) {
		fxmessage("Support for mo3 enabled\n");
		_unmo3=true;
		_ext.append(",*.mo3");
	} else {
		fxmessage("Support for mo3 disabled\n");
		_unmo3=false;
	}

	loadSettings();

	setSampleSize();
}

modBackend::~modBackend() {

	close();
}

void modBackend::loadSettings() {

	_settings.samplerate=load("Frequency", 44100);
	_settings.channels=load("Channels", 2);
	_settings.precision=(load("Bits", 16)==16)?S16:U8;
	_settings.oversampling=load("Oversampling", TRUE);
	_settings.noiseRed=load("Noise reduction", FALSE);
	_settings.resamplingMode=load("Resampling", SRCMODE_SPLINE);
	_settings.reverbDepth=load("Rev depth", 0);
	_settings.reverbDelay=load("Rev delay", 40);
	_settings.bassAmount=load("Bass amount", 0);
	_settings.bassRange=load("Bass range", 10);
	_settings.surroundDepth=load("Surr depth", 0);
	_settings.surroundDelay=load("Surr delay", 5);
}

void modBackend::saveSettings() {

	setSampleSize();

	save("Frequency", _settings.samplerate);
	save("Channels", _settings.channels);
	save("Bits", (_settings.precision==S16)?16:8);
	save("Oversampling", _settings.oversampling);
	save("Noise reduction", _settings.noiseRed);
	save("Resampling", _settings.resamplingMode);
	save("Rev depth", _settings.reverbDepth);
	save("Rev delay", _settings.reverbDelay);
	save("Bass amount", _settings.bassAmount);
	save("Bass range", _settings.bassRange);
	save("Surr depth", _settings.surroundDepth);
	save("Surr delay", _settings.surroundDelay);
}

bool modBackend::open(const FXString& fileName) {

	close();

	bool tmpFile=false;
	FXString fName;

	if (_unmo3 && (!comparecase(FXPath::extension(fileName), "mo3"))) {
		FXString command=FXStringFormat("%s -s %s ", UNMO3, FXPath::enquote(fileName).text());
		fName=tempFile(fileName);
		command.append(FXPath::enquote(fName));
		FXTRACE((10, "Command: %s\n", command.text()));
		runCmd(command);
		FXTRACE((10, "Done\n"));
		tmpFile=true;
	}
#ifdef HAVE_LIBZ
	else if (FXPath::match(EXTZ, FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD)) {
#ifdef _WIN32
		zlib_filefunc64_def ffunc;
		fill_win32_filefunc64(&ffunc);
		unzFile modZip=unzOpen2_64(convertUtf(fileName), &ffunc);
#else
		unzFile modZip=unzOpen64(fileName.text());
#endif
		if (unzGoToFirstFile(modZip)!=UNZ_OK) {
			unzClose(modZip);
			return false;
		}
		fName=tempFile(fileName);
		FXFile modFile(fName, FXFile::Writing);
		unzOpenCurrentFile(modZip);
		FXint n;
		FXchar buffer[4096];
		do {
			n=unzReadCurrentFile(modZip, buffer, 4096);
		} while (modFile.writeBlock(buffer, n));
		unzCloseCurrentFile(modZip);
		unzClose(modZip);
		tmpFile=true;
	}
#endif

	if(!tmpFile)
		fName=fileName;

#if defined(HAVE_MMAP) || defined(_WIN32)
	FXMemMap memMap;
	const char* data=(char*)memMap.mapFile(fName);
	if (!data) {
		delTempFile(tmpFile, fName);
		return false;
	}
	const long size=memMap.length();
#else
	FXFile f(fName);
	const long size=f.size();
	const char* data=new char[size];
	f.readBlock((void*)data, size);
	f.close();
#endif

	_module=new CSoundFile();
	const bool res=_module->Create((const BYTE*)data, size);

#if defined(HAVE_MMAP) || defined(_WIN32)
	memMap.unmap();
#else
	delete [] data;
#endif

	delTempFile(tmpFile, fName);

	if (!res) {
		delPtr(_module);
		return false;
	}

	_module->SetReverbParameters(_settings.reverbDepth, _settings.reverbDelay);
	_module->SetXBassParameters(_settings.bassAmount, _settings.bassRange);
	_module->SetSurroundParameters(_settings.surroundDepth, _settings.surroundDelay);
	_module->SetWaveConfig(_settings.samplerate, (_settings.precision==S16)?16:8, _settings.channels, _mmx);
	_module->SetWaveConfigEx(_settings.surroundDepth, !_settings.oversampling,
		_settings.reverbDepth, true, _settings.bassAmount,
		_settings.noiseRed, false);
	_module->SetResamplingMode(_settings.resamplingMode);

	_metaData.addInfo(metaData::TITLE, _module->GetTitle());
	_metaData.addInfo(metaData::COMMENT, _module->m_lpszSongComments);

	time(_module->GetSongTime());

	songLoaded(fileName.text());
	return true;
}

void modBackend::close() {

	if (_module) {
		_module->Destroy();
		delPtr(_module);
	}

	songLoaded(0);
}

bool modBackend::rewind() {

	if (_module) {
		_module->SetCurrentPos(0);
		return true;
	}
	return false;
}

/*****************************************************************/

#define SETTINGS	modBackend::_settings

FXDEFMAP(modConfig) modConfigMap[] = {
	// Message_Type		ID				Message_Handler
	FXMAPFUNC(SEL_COMMAND,	modConfig::ID_FREQUENCY,	modConfig::onCmdFrequency),
	FXMAPFUNC(SEL_COMMAND,	modConfig::ID_CHANNELS,		modConfig::onCmdChannels),
	FXMAPFUNC(SEL_COMMAND,	modConfig::ID_BITS,		modConfig::onCmdBits),
	FXMAPFUNC(SEL_COMMAND,	modConfig::ID_RESAMPLING,	modConfig::onCmdResampling),
};

FXIMPLEMENT(modConfig, configFrame, modConfigMap, ARRAYNUMBER(modConfigMap))

modConfig::modConfig(FXPacker* win) :
	configFrame(win, modBackend::name, CREDITS, LINK) {

	FXint val;

	new FXLabel(matrix(), tr("Frequency"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *freqBox=new FXListBox(matrix(), this, modConfig::ID_FREQUENCY, LAYOUT_FILL|FRAME_NORMAL);
	freqBox->fillItems("11025\n22050\n44100\n48000");
	freqBox->setNumVisible(4);
	switch (SETTINGS.samplerate) {
	case 11025:
		val=0;
		break;
	case 22050:
		val=1;
		break;
	default:
	case 44100:
		val=2;
		break;
	case 48000:
		val=3;
		break;
	}
	freqBox->setCurrentItem(val);

	new FXLabel(matrix(), tr("Channels"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *chanBox=new FXListBox(matrix(), this, modConfig::ID_CHANNELS, LAYOUT_FILL|FRAME_NORMAL);
	chanBox->fillItems("Mono\nStereo");
	chanBox->setNumVisible(2);
	chanBox->setCurrentItem(SETTINGS.channels-1);

	new FXLabel(matrix(), tr("Bits"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *bitBox=new FXListBox(matrix(), this, modConfig::ID_BITS, LAYOUT_FILL|FRAME_NORMAL);
	bitBox->fillItems("8\n16");
	bitBox->setNumVisible(2);
	switch (SETTINGS.precision) {
	case U8:
		val=0;
		break;
	default:
	case S16:
		val=1;
		break;
	/*case S24:
		val=2;
		break;
	case S32:
		val=3;
		break;*/
	}
	bitBox->setCurrentItem(val);

	new FXLabel(matrix(), tr("Resampling"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *resBox=new FXListBox(matrix(), this, modConfig::ID_RESAMPLING, LAYOUT_FILL|FRAME_NORMAL);
	resBox->fillItems("No resampling\nLinear\nCubic Spline\nHigh quality");
	resBox->setNumVisible(4);
	switch (SETTINGS.resamplingMode) {
	case SRCMODE_NEAREST:
		val=0;
		break;
	case SRCMODE_LINEAR:
		val=1;
		break;
	default:
	case SRCMODE_SPLINE:
		val=2;
		break;
	case SRCMODE_POLYPHASE:
		val=3;
		break;
	}
	resBox->setCurrentItem(val);

	new FXVerticalSeparator(extraLeft());

	FXVerticalFrame *vert=new FXVerticalFrame(extraLeft());
	new FXCheckButton(vert, tr("Oversampling"), &_oversampling, FXDataTarget::ID_VALUE);
	new FXCheckButton(vert, tr("Noise reduction"), &_noiseRed, FXDataTarget::ID_VALUE);

	//
	FXHorizontalFrame *hf=new FXHorizontalFrame(extraBottom(), 0, 0,0,0,0, 0,0,0,0, 0,0);

	_oversampling.connect(SETTINGS.oversampling);
	_noiseRed.connect(SETTINGS.noiseRed);
	_revDepth.connect(SETTINGS.reverbDepth);
	_revDelay.connect(SETTINGS.reverbDelay);
	_bassAmount.connect(SETTINGS.bassAmount);
	_bassRange.connect(SETTINGS.bassRange);
	_surrDepth.connect(SETTINGS.surroundDepth);
	_surrDelay.connect(SETTINGS.surroundDelay);

	FXKnob		*knob;
	FXVerticalFrame	*vf;
	FXTextField	*tf;
	FXMatrix	*mat;

	vert=new FXVerticalFrame(hf, FRAME_GROOVE);
	new FXLabel(vert, tr("Reverb"), 0,  LABEL_NORMAL|LAYOUT_FILL_X);
	mat=new FXMatrix(vert, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL|PACK_UNIFORM_WIDTH);
	vf=new FXVerticalFrame(mat);
	new FXLabel(vf, tr("Depth"));
	tf=new FXTextField(vf, 3, &_revDepth, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL|LAYOUT_CENTER_X|JUSTIFY_RIGHT);
	tf->disable();
	new FXKnob(mat, &_revDepth, FXDataTarget::ID_VALUE, LAYOUT_CENTER_Y|KNOB_NORMAL);
	vf=new FXVerticalFrame(mat);
	new FXLabel(vf, tr("Delay"));
	tf=new FXTextField(vf, 3, &_revDelay, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL|LAYOUT_CENTER_X|JUSTIFY_RIGHT);
	tf->disable();
	knob=new FXKnob(mat, &_revDelay, FXDataTarget::ID_VALUE, LAYOUT_CENTER_Y|KNOB_NORMAL);
	knob->setRange(40, 200);

	vert=new FXVerticalFrame(hf, FRAME_GROOVE);
	new FXLabel(vert, tr("Bass exp"), 0,  LABEL_NORMAL|LAYOUT_FILL_X);
	mat=new FXMatrix(vert, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL|PACK_UNIFORM_WIDTH);
	vf=new FXVerticalFrame(mat);
	new FXLabel(vf, tr("Amount"));
	tf=new FXTextField(vf, 3, &_bassAmount, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL|LAYOUT_CENTER_X|JUSTIFY_RIGHT);
	tf->disable();
	new FXKnob(mat, &_bassAmount, FXDataTarget::ID_VALUE, LAYOUT_CENTER_Y|KNOB_NORMAL);
	vf=new FXVerticalFrame(mat);
	new FXLabel(vf, tr("Range"));
	tf=new FXTextField(vf, 3, &_bassRange, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL|LAYOUT_CENTER_X|JUSTIFY_RIGHT);
	tf->disable();
	knob=new FXKnob(mat, &_bassRange, FXDataTarget::ID_VALUE, LAYOUT_CENTER_Y|KNOB_NORMAL);
	knob->setRange(10, 100);

	vert=new FXVerticalFrame(hf, FRAME_GROOVE);
	new FXLabel(vert, tr("Surround"), 0,  LABEL_NORMAL|LAYOUT_FILL_X);
	mat=new FXMatrix(vert, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL|PACK_UNIFORM_WIDTH);
	vf=new FXVerticalFrame(mat);
	new FXLabel(vf, tr("Depth"));
	tf=new FXTextField(vf, 3, &_surrDepth, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL|LAYOUT_CENTER_X|JUSTIFY_RIGHT);
	tf->disable();
	new FXKnob(mat, &_surrDepth, FXDataTarget::ID_VALUE, LAYOUT_CENTER_Y|KNOB_NORMAL);
	vf=new FXVerticalFrame(mat);
	new FXLabel(vf, tr("Delay"));
	tf=new FXTextField(vf, 3, &_surrDelay, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL|LAYOUT_CENTER_X|JUSTIFY_RIGHT);
	tf->disable();
	knob=new FXKnob(mat, &_surrDelay, FXDataTarget::ID_VALUE, LAYOUT_CENTER_Y|KNOB_NORMAL);
	knob->setRange(5, 40);
}

long modConfig::onCmdFrequency(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		SETTINGS.samplerate=11025;
		break;
	case 1:
		SETTINGS.samplerate=22050;
		break;
	case 2:
		SETTINGS.samplerate=44100;
		break;
	case 3:
		SETTINGS.samplerate=48000;
		break;
		//TODO if libmodplug >= 0.8 add also 96KHz and 192KHz
	}
	return 1;
}

long modConfig::onCmdChannels(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	SETTINGS.channels=val+1;
	return 1;
}

long modConfig::onCmdBits(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		SETTINGS.precision=U8;
		break;
	case 1:
		SETTINGS.precision=S16;
		break;
	/*case 2:
		SETTINGS.precision=S24;
		break;
	case 3:
		SETTINGS.precision=S32;
		break;*/
	}
	return 1;
}

long modConfig::onCmdResampling(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		SETTINGS.resamplingMode=SRCMODE_NEAREST;
		break;
	case 1:
		SETTINGS.resamplingMode=SRCMODE_LINEAR;
		break;
	case 2:
		SETTINGS.resamplingMode=SRCMODE_SPLINE;
		break;
	case 3:
		SETTINGS.resamplingMode=SRCMODE_POLYPHASE;
		break;
	}
	return 1;
}

#endif
