/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef MOD_H
#define MOD_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_LIBMODPLUG
#include <libmodplug/stdafx.h>
#include <libmodplug/sndfile.h>

#include "inputBackend.h"
#include <fx.h>
#if defined(HAVE_MMAP) || defined(_WIN32)
#  include <FXMemMap.h>
#endif

/*****************************************************************/

typedef struct {
	FXbool		oversampling;
	FXbool		noiseRed;
	FXint		channels;
	sample_t	precision;
	FXint		samplerate;
	FXint		resamplingMode;
	FXint		reverbDepth;
	FXint		reverbDelay;
	FXint		bassAmount;
	FXint		bassRange;
	FXint		surroundDepth;
	FXint		surroundDelay;
} modConfig_t;

/*****************************************************************/

#include "configFrame.h"

class modConfig : public configFrame {
	FXDECLARE(modConfig)

private:
	FXDataTarget	_oversampling;
	FXDataTarget	_noiseRed;
	FXDataTarget	_revDepth;
	FXDataTarget	_revDelay;
	FXDataTarget	_bassAmount;
	FXDataTarget	_bassRange;
	FXDataTarget	_surrDepth;
	FXDataTarget	_surrDelay;

private:
	modConfig() {};
	modConfig(const modConfig&);
	modConfig& operator=(const modConfig&);

public:
	modConfig(FXPacker* win);
	virtual ~modConfig() {};

public:
	enum{
		ID_FREQUENCY=FXDialogBox::ID_LAST,
		ID_CHANNELS,
		ID_BITS,
		ID_RESAMPLING,
		ID_REVERB,
		ID_MEGABASS,
		ID_SURROUND,
		ID_LAST,
	};

public:
	long onCmdFrequency(FXObject*, FXSelector, void*);
	long onCmdChannels(FXObject*, FXSelector, void*);
	long onCmdBits(FXObject*, FXSelector, void*);
	long onCmdResampling(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class modBackend : public inputBackend {

	friend class modConfig;

private:
	static modConfig_t	_settings;

private:
	CSoundFile	*_module;

	unsigned int	_sampleSize;

	const bool	_mmx;
	bool		_unmo3;

	FXString	_ext;

private:
	modBackend();

	/// Get temp file name
	const FXString tempFile(const FXString& fileName) {
		return FXSystem::getTempDirectory()+PATHSEP+FXPath::title(fileName)+".mod";
	}

	/// Delete temp file
	void delTempFile(const bool del, const FXString& tmpFile) {
		if (del) {
			FXTRACE((10, "Deleting temp file: %s\n", tmpFile.text()));
			FXFile::remove(tmpFile);
		}
	}

	void setSampleSize() {
		_sampleSize=_settings.channels*((_settings.precision==S16)?2:1);
	}

public:
	~modBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new modBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const { return _ext; }

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get samplerate
	unsigned int samplerate() const { return _settings.samplerate; }

	/// Get channels
	unsigned int channels() const { return _settings.channels; }

	/// Get precision
	sample_t precision() const { return _settings.precision; }

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new modConfig(win); }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
