/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef OGG_H
#define OGG_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_VORBIS

#include "inputBackend.h"

#include <fx.h>
#include <vorbis/vorbisfile.h>

/*****************************************************************/

typedef struct {
	sample_t	precision;
} oggConfig_t;

/*****************************************************************/

#include "configFrame.h"

class oggConfig : public configFrame {
	FXDECLARE(oggConfig)

private:
	oggConfig() {};
	oggConfig(const oggConfig&);
	oggConfig& operator=(const oggConfig&);

public:
	oggConfig(FXPacker* win);
	virtual ~oggConfig() {};

public:
	enum{
		ID_BITS=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdBits(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class oggBackend : public inputBackend {

	friend class oggConfig;

private:
	static oggConfig_t	_settings;

private:
	OggVorbis_File	*_vf;
	vorbis_info	*_vi;

	FXFile		_file;

private:
	static ov_callbacks	vorbis_callbacks;

	static size_t	read_func(void *ptr, size_t size, size_t nmemb, void *datasource);
	static int	seek_func(void *datasource, ogg_int64_t offset, int whence);
	static int	close_func(void *datasource);
	static long	tell_func(void *datasource);

private:
	oggBackend();

	bool getComment(const char* orig, FXString* dest, const char* type);

public:
	~oggBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new oggBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get samplerate
	unsigned int samplerate() const { return _vi?_vi->rate:0; }

	/// Get channels
	unsigned int channels() const { return _vi?_vi->channels:0; }

	/// Get precision
	sample_t precision() const { return _settings.precision; }

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Gapless support
	bool gapless() const { return true; };

	/// Open config dialog
	void config(FXPacker* win) { new oggConfig(win); }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
