/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "sndBackend.h"

#ifdef HAVE_SNDFILE

#define CREDITS		"libsndfile\nCopyright \302\251 Erik de Castro Lopo"
#define LINK		"http://www.mega-nerd.com/libsndfile/"

FXString sndBackend::_ext="*.(";

const char sndBackend::name[]="Sndfile";

/*****************************************************************/

size_t sndBackend::fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds) {

	return (sf_read_short(_sf, (short*)buffer, (bufferSize>>1))<<1);
}

/*****************************************************************/

bool sndBackend::init() {

	FXuint count;
	sf_command(0, SFC_GET_SIMPLE_FORMAT_COUNT, &count, sizeof(int));

	SF_FORMAT_INFO format_info;
	for (FXuint i=0; i<count; i++) {
		format_info.format=i;
		sf_command(0, SFC_GET_SIMPLE_FORMAT, &format_info, sizeof(format_info));
		if (_ext.find(format_info.extension)<0) {
			_ext.append(format_info.extension);
			_ext.append('|');
		}
	}

	_ext.replace(_ext.length()-1, ')');

	FXTRACE((10, "Sndfile extensions supported: %s\n", _ext.text()));
	return true;
}

bool sndBackend::supports(const char* fileName) {

#if FOX_MINOR>6
	return FXPath::match(FXPath::name(fileName), _ext, NoEscape|PathName|CaseFold);
#else
	return FXPath::match(_ext, FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
#endif
}

sndBackend::~sndBackend() {

	close();
}

bool sndBackend::open(const FXString& fileName) {

	close();

	_si.format=0;
#if defined (_WIN32) && defined (UNICODE)
	const FXint n=ncslen(fileName.text(), fileName.length());
	FXnchar *buffer=new FXnchar[n+1];
	utf2ncs(buffer, fileName.text(), fileName.length()+1);

#ifdef ENABLE_SNDFILE_WINDOWS_PROTOTYPES
	_sf=sf_wchar_open(buffer, SFM_READ, &_si);
#else
	_sf=sf_open((char*)buffer, SFM_READ, &_si);
#endif

	delete [] buffer;
#else
	_sf=sf_open(fileName.text(), SFM_READ, &_si);
#endif
	if (!_sf) {
		fxmessage("%s\n", sf_strerror(0));
		return false;
	}

	sf_command(_sf, SFC_SET_SCALE_FLOAT_INT_READ, 0, SF_TRUE);

	_metaData.addInfo(metaData::TITLE, sf_get_string(_sf, SF_STR_TITLE));
	_metaData.addInfo(metaData::ARTIST, sf_get_string(_sf, SF_STR_ARTIST));
	_metaData.addInfo(gettext("copyright"), sf_get_string(_sf, SF_STR_COPYRIGHT));
	_metaData.addInfo(metaData::COMMENT, sf_get_string(_sf, SF_STR_COMMENT));

	const int seconds=_si.frames/_si.samplerate;
	time((seconds>0x7FFFFFFF)?0:seconds);

	songLoaded(fileName.text());
	return true;
}

void sndBackend::close() {

	if (_sf) {
		sf_close(_sf);
		_sf=0;
	}

	songLoaded(0);
}

bool sndBackend::rewind() {

	if (!_sf)
		return false;

	if (sf_seek(_sf, 0, SEEK_SET)<0) {
		fxmessage("%s\n", sf_strerror(_sf));
		return false;
	}

	return true;
}

/*****************************************************************/

FXIMPLEMENT(sndConfig, configFrame, 0, 0)

sndConfig::sndConfig(FXPacker* win) :
	configFrame(win, sndBackend::name, CREDITS, LINK) {

	new FXLabel(matrix(), tr("No settings available"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
}

#endif
