/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SND_H
#define SND_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_SNDFILE

#ifdef ENABLE_SNDFILE_WINDOWS_PROTOTYPES
#  include <windows.h>
#endif

#include <sndfile.h>

#include "inputBackend.h"
#include <fx.h>

/*****************************************************************/

#include "configFrame.h"

class sndConfig : public configFrame {
	FXDECLARE(sndConfig)

private:
	sndConfig() {};
	sndConfig(const sndConfig&);
	sndConfig& operator=(const sndConfig&);

public:
	sndConfig(FXPacker* win);
	virtual ~sndConfig() {};
};

/*****************************************************************/

class sndBackend : public inputBackend {

private:
	SNDFILE		*_sf;
	SF_INFO		_si;

	static FXString		_ext;

private:
	sndBackend() :
		inputBackend(name),
		_sf(0) {};

public:
	~sndBackend();

	static const char name[];

	static bool init();

	/// Factory method
	static input* factory() { return new sndBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const { return _ext; }

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get samplerate
	unsigned int samplerate() const { return _sf?_si.samplerate:0; }

	/// Get channels
	unsigned int channels() const { return _sf?_si.channels:0; }

	/// Get precision
	sample_t precision() const { return S16; }

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Gapless support
	bool gapless() const { return true; };

	/// Open config dialog
	void config(FXPacker* win) { new sndConfig(win); }
};

#endif

#endif
