/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef WMM_H
#define WMM_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef _WIN32

#include <windows.h>
#include <mmsystem.h>

#include <fx.h>

#include "outputBackend.h"

/*****************************************************************/

/**
 * WMM output backend
*/
class wmmBackend : public outputBackend {

private:
	HWAVEOUT	_wHandle;
	WAVEHDR		_wHdr[2];
	HANDLE		_event[2];
	unsigned int	_idx;

private:
	static void CALLBACK waveOutProc(HWAVEOUT waveOut, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);
	static const char* checkErr(const MMRESULT err);

private:
	wmmBackend();

public:
	virtual ~wmmBackend() {};

	static const char name[];

	/// Factory method
	static output* factory() { return new wmmBackend(); }

	/// Open
	size_t open(const unsigned int card, unsigned int &sampleRate, const unsigned int channels, const unsigned int prec);

	/// Close
	void close();

	/// Get buffer
	void *buffer() { return _wHdr[_idx].lpData; }

	/// Write data to output
	bool write(void* buffer, size_t bufferSize);

	/// Pause
	void pause() { stop(); }

	/// Stop
	void stop();

	/// Set volume
	void volume(int vol);

	/// Get volume
	int volume();
};

#endif

#endif
