/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <fx.h>

#include "bookmark.h"
#include "iconFactory.h"

#if FOX_MINOR>6
#  define SECTION	FXStringFormat("%s Bookmarks", _type)
#  define KEY(i)	FXStringFormat("bm%d", i)
#else
#  define SECTION	FXStringFormat("%s Bookmarks", _type).text()
#  define KEY(i)	FXStringFormat("bm%d", i).text()
#endif

FXDEFMAP(bookmark) bookmarkMap[] = {
	FXMAPFUNC(SEL_DND_MOTION,		0,	bookmark::onDNDMotion),
	FXMAPFUNC(SEL_DND_DROP,			0,	bookmark::onDNDDrop),
	FXMAPFUNC(SEL_COMMAND,			0,	bookmark::onCmdGet),
	FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,	0,	bookmark::onRgtClk),
	FXMAPFUNC(SEL_COMMAND,	bookmark::ID_BM_DEL,	bookmark::onCmdDel),
	FXMAPFUNC(SEL_COMMAND,	bookmark::ID_BM_CLEAR,	bookmark::onCmdClear),
};

FXIMPLEMENT(bookmark, list, bookmarkMap, ARRAYNUMBER(bookmarkMap))

void bookmark::load(const char* type) {

	_type=type;

	const FXchar* bm;
	int i=0;
	while ((bm=getApp()->reg().readStringEntry(SECTION, KEY(i++))))
		appendItem(bm);
}

void bookmark::save() {

	getApp()->reg().deleteSection(SECTION);

	const FXint n=getNumItems();
	if (!n)
		return;

	for (int i=0; i<n; i++)
		getApp()->reg().writeStringEntry(SECTION, KEY(i), getItemText(i).text());

	clearItems();
}

void bookmark::add(const FXString& dirName) {

	if (findItem(dirName)==-1)
		appendItem(dirName);
}

long bookmark::onDNDDrop(FXObject*, FXSelector, void*) {

	FXString item;
	getDNDData(FROM_DRAGNDROP, urilistType, item);
	dropFinished(DRAG_COPY);

	add(FXURL::fileFromURL(item));

	return 1;
}

long bookmark::onDNDMotion(FXObject*, FXSelector, void* ptr) {

	FXString item;
	acceptDrop(getDNDData(FROM_DRAGNDROP, urilistType, item) && FXStat::isDirectory(FXURL::fileFromURL(item))?
		DRAG_COPY:DRAG_REJECT);

	return 1;
}

long bookmark::onRgtClk(FXObject* sender, FXSelector sel, void* ptr) {

	list::onRightBtnRelease(sender, sel, ptr);
	FXEvent *event=(FXEvent*)ptr;
	if (event->moved)
		return 1;

	const FXint item=getItemAt(event->win_x, event->win_y);
	setCurrentItem(item);

	FXMenuPane pane(this);
	FXString dirName=(item>=0)?getItemText(item):tr("No item selected");
	if (dirName.count()>20)
		dirName=dirName.left(10)+"..."+dirName.right(10);
	new FXLabel(&pane, dirName);
	new FXMenuSeparator(&pane);
	FXMenuCommand* mc=new FXMenuCommand(&pane, tr("Delete\t\tDelete selected bookmark"),
		GET_ICON(icon_listremove), this, bookmark::ID_BM_DEL);
	if (item<0)
		mc->disable();
	mc=new FXMenuCommand(&pane, tr("Clear\t\tClear all bookmarks"),
		GET_ICON(icon_editdelete), this, bookmark::ID_BM_CLEAR);
	if (getNumItems()==0)
		mc->disable();
	new FXMenuCommand(&pane, tr("Cancel\t\tCancel action"), GET_ICON(icon_actioncancel));
	pane.create();
	pane.popup(0, event->root_x, event->root_y);
	getApp()->runModalWhileShown(&pane);
	return 1;
}

long bookmark::onCmdGet(FXObject*, FXSelector, void* data) {

	const FXString tmp=getItemText((FXint)(FXival)data);
	return target->handle(this, FXSEL(SEL_COMMAND, message), (void*)&tmp);
}

long bookmark::onCmdDel(FXObject*, FXSelector, void*) {

	const FXint idx=getCurrentItem();
	if (idx<0)
		return 0;
	removeItem(idx);
	return 1;
}

long bookmark::onCmdClear(FXObject*, FXSelector, void*) {

	if (FXMessageBox::question(this, MBOX_YES_NO, tr("Clear bookmarks"),
	tr("Are you sure you want to delete all bookmarks?"))==MBOX_CLICKED_YES) {
		clearItems();
		return 1;
	}
	return 0;
}
