/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef INFODIALOG_H
#define INFODIALOG_H

#include <fx.h>

class metaData;

class imageLoader : public FXThread {

private:
	FXString	_name;
	FXGUISignal	*_loaded;

protected:
	FXint run();

public:
	imageLoader(FXGUISignal *loaded)
		: FXThread(), _loaded(loaded) {};
	~imageLoader() { delete _loaded; }

	/// Load an image
	FXbool load(const FXString& file);
};

/*****************************************************************/

class infoDialog : public FXDialogBox {
	FXDECLARE(infoDialog)

private:
	FXFont		*_font;
	FXFont		*_italicFont;
	imageLoader	*_imageLoader;
	FXImageFrame	*_imgFrame;
	FXMatrix	*_matrix;
	FXText		*_text;
	FXPacker	*_textFrame;

private:
	infoDialog() {};
	infoDialog(const infoDialog&);
	infoDialog& operator=(const infoDialog&);

	void setNewImage(FXImage*);

public:
	infoDialog(FXWindow*);
	virtual ~infoDialog();

	void setInfo(const metaData*);

public:
	enum {
		ID_IMG_LOADED=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdImgLoaded(FXObject*, FXSelector, void*);
};

#endif
