/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef LIST_H
#define LIST_H

#include <fx.h>

class listItem : public FXListItem {
	FXDECLARE(listItem)

	friend class list;

protected:
	FXString  _text;

private:
	listItem(const listItem&);
	listItem& operator=(const listItem&);

	const FXString transform(const FXString& text) const { return FXPath::title(text).substitute('_', ' '); }

protected:
	listItem() {};
	virtual void draw(const FXList* list, FXDC& dc, FXint x, FXint y, FXint h) const;

public:
	/// Construct new item with given text, icon, and user-data
	listItem(const FXString& text, FXIcon* ic=0, void* ptr=0) :
		FXListItem(text, ic, ptr),
		_text(transform(text)) {}

	/// Return width of item as drawn in list
	virtual FXint getWidth(const FXList* list) const;

	/// Change item's text label
	virtual void setText(const FXString& txt);
};

/*****************************************************************/

class list : public FXList {
	FXDECLARE(list)

public:
	FXColor	altbackColor;

private:
	list(const list&);
	list& operator=(const list&);

protected:
	list() {};
	virtual listItem* createItem(const FXString& text, FXIcon* icon, void* ptr);

public:
	/// Construct a list with initially no items in it
	list(FXComposite* p, FXObject* tgt, FXSelector sel, FXuint opts=LIST_NORMAL);

	/// Destructor
	virtual ~list() {};

	/// Return alternate text background
	FXColor getAltBackColor() const { return altbackColor; }

	/// Change alternate text background
	void setAltBackColor(FXColor clr);

public:
	long onPaint(FXObject*, FXSelector, void*);
};

#endif
