/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef PLAYLIST_H
#define PLAYLIST_H

#include <fx.h>
#include "list.h"

#include "trackList/trackList.h"

class playlist : public list {
	FXDECLARE(playlist)

private:
	static FXuint	_seed;

	tracks	*_tracks;

private:
	playlist() {};
	playlist(const playlist&);
	playlist& operator=(const playlist&);

	static FXint ascending(const FXListItem* a, const FXListItem* b);
	static FXint descending(const FXListItem* a, const FXListItem* b);
	static FXint random(const FXListItem* a, const FXListItem* b);

public:
	playlist(FXComposite* p, FXObject* tgt=0, FXSelector sel=0) :
		list(p, tgt, sel, LAYOUT_FILL|LIST_BROWSESELECT),
		_tracks(0) {}
	~playlist() { delete _tracks; }

	/// Add item to playlist
	bool add(const FXString& item);

	/// Load playlist
	int load(const FXString& path);

	/// Save playlist
	bool save(const FXString& file);

	/// Filter playlist
	int filter(const FXString& filter);

	/// Set sorting type
	void sorting(const char* sort);

	/// Get sorting type
	const char* sorting();

	/// Get track location
	const FXString getLocation(FXint index) { return ((track_t*)getItemData(index))->location(); }

	/// Get track start
	const int getStart(FXint index) { return ((track_t*)getItemData(index))->start(); }

public:
	enum {
		ID_ITEM_DEL=list::ID_LAST,
		ID_SORT_ASC,
		ID_SORT_DESC,
		ID_SORT_RANDOM,
		ID_LAST,
	};

public:
	long onDNDDrop(FXObject*, FXSelector, void*);
	long onDNDMotion(FXObject*, FXSelector, void*);
	long onRgtClk(FXObject*, FXSelector, void*);
	long onCmdDel(FXObject*, FXSelector, void*);
	long onCmdSort(FXObject*, FXSelector, void*);
	long onUpdSort(FXObject*, FXSelector, void*);
};

#endif
