/*
 *  Copyright (C) 2008-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SETTINGS_H
#define SETTINGS_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <fx.h>

class inputConfig;

class settingsWindow : public FXDialogBox {
	FXDECLARE(settingsWindow)

private:
	FXDataTarget	_subtunes;
	FXDataTarget	_autoBk;
	FXDataTarget	_albumArtQuality;
	FXDataTarget	_replayGainMode;
	FXVerticalFrame	*_replayGainBox;
#ifdef _WIN32
	FXCheckButton	*_cb;
	FXDataTarget	_wlm;
#endif
#ifdef HAVE_BS2B
	FXDataTarget	_bs2b;
#endif

private:
	settingsWindow() {};
	settingsWindow(const settingsWindow&);
	settingsWindow& operator=(const settingsWindow&);

	void enableControls(FXPacker* p, const FXbool enable);

public:
	settingsWindow(FXWindow* win, inputConfig* i);
	virtual ~settingsWindow() {};

public:
	enum {
		ID_COLOR=FXDialogBox::ID_LAST,
		ID_REPLAYGAIN,
#ifdef _WIN32
		ID_MSN,
#endif
		ID_LAST,
	};

public:
	long onCmdColor(FXObject*, FXSelector, void*);
	long onCmdReplayGain(FXObject*, FXSelector, void*);
#ifdef _WIN32
	long onCmdMessenger(FXObject*, FXSelector, void*);
#endif
};

/*****************************************************************/

#define SETTINGS	settings::instance()

class settings {

	friend class settingsWindow;
	friend class audioConfig;

private:
	FXString _apiName;
	FXString _card;

	FXbool	_subtunes;
	FXbool	_autoBk;
	FXColor	_altCol;
	FXbool	_albumArtQuality;
	FXbool	_replayGain;
	FXint	_replayGainMode;
#ifdef _WIN32
	FXbool	_nowPlaying;
	FXbool	_wlm;
#endif
#ifdef HAVE_BS2B
	FXbool	_bs2b;
#endif
	FXuint	_bits;

protected:
	settings() {};
	settings(const settings&);
	settings& operator= (const settings&);
	~settings() {};

public:
	/// Get singleton instance
	static settings* instance();

	/// Load setting
	void load();

	/// Save settings
	void save();

	// Get program settings

	/// Audio API
	const char* apiName() { return _apiName.text(); };

	/// Soundcard
	const char* card() { return _card.text(); };

	/// Play subtunes
	FXbool subtunes() { return _subtunes; }

	/// Automatic backend selection
	FXbool autoBk() { return _autoBk; }

	/// Alternate color for lists
	FXColor altCol() { return _altCol; }

	/// Show album art
	FXbool albumArtQuality() { return _albumArtQuality; }

	/// Replay Gain
	FXbool replayGain() { return _replayGain; }

	/// Replay Gain Mode
	FXint replayGainMode() { return _replayGainMode; }

#ifdef _WIN32
	/// Windows Messenger Now Playing
	FXbool nowPlaying() { return _nowPlaying; }

	/// Windows Live Messanger
	FXbool wlm() { return _wlm; }
#endif
#ifdef HAVE_BS2B
	/// Bauer stereophonic-to-binaural DSP
	FXbool bs2b() { return _bs2b; }
#endif
	/// Default bittdepth
	FXuint bits() { return _bits; }
};

#endif
