/*
 *  Copyright (C) 2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

// Originally based on code posted by Rafael de Pelegrini Soares on the Fox-toolkit mailing list

#include "textLabel.h"

#ifdef _WIN32
#  include <windows.h>
#endif

FXDEFMAP(textLabel) textLabelMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,	0,	textLabel::onLeftBtnPress),
  FXMAPFUNC(SEL_ENTER,			0,	textLabel::onEnter),
  FXMAPFUNC(SEL_LEAVE,			0,	textLabel::onLeave),
  FXMAPFUNC(SEL_TIMEOUT,	textLabel::ID_TIMER,	textLabel::onTimer),
};

FXIMPLEMENT(textLabel, FXLabel, textLabelMap, ARRAYNUMBER(textLabelMap))

bool textLabel::fxexecute(const FXString& link) {
#ifdef _WIN32
#ifdef UNICODE
	FXint n=ncslen(link.text(), link.length());
	FXnchar *buffer=new FXnchar[n+1];
	utf2ncs(buffer,link.text(), link.length()+1);
	n=((int)ShellExecute(NULL, L"open", buffer, NULL, NULL, SW_SHOW)>32);
	delete [] buffer;
	return n;
#else
	return ((int)ShellExecute(NULL, "open", link.text(), NULL, NULL, SW_SHOW)>32);
#endif // UNICODE
#else
	const FXString list="opera\tkonqueror\tmozilla-firefox\tmozilla\tnetscape\tdillo\tlynx";

	const FXString path=FXSystem::getExecPath();
	FXint index=0;

	FXString software=list.section("\t", index);
	while (!software.empty()) {
		software=FXPath::search(path, software);
		if (!software.empty())
			return (system(FXStringFormat("%s \"%s\" >/dev/null 2>&1 & ", software.text(), link.text()).text())>=0);
		software=list.section("\t", ++index);
	}

	return false;
#endif
}

textLabel::textLabel(FXComposite* p, const FXString& text, FXuint opts, const FXint maxLength) :
	FXLabel(p, FXString::null, 0, opts),
	_link(FXString::null) {

	if (FXRex("\\A(?ihttp://)?(?iwww\\.)?[-\\w/~][-\\w/~\\.]+\\.\\l{2,4}(/[-\\w~\\.]*)*\\Z").match(text)) {
		_link=text;
		setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
		setTextColor(FXRGB(0, 0, 255));
	}

	label=text;
	if (maxLength && text.count()>maxLength) {
		const int pos=label.rfind(' ', maxLength);
		label.replace(pos>0?pos:maxLength, '\n');
	}
}

textLabel::~textLabel() {

	getApp()->removeTimeout(this, ID_TIMER);
}

long textLabel::onLeftBtnPress(FXObject*, FXSelector, void*) {

	if (!_link.empty()) {
		getApp()->beginWaitCursor();
		if (fxexecute(_link))
			getApp()->addTimeout(this, ID_TIMER, 2000); // 2 seconds of wait cursor
		else {
			getApp()->endWaitCursor();
			getApp()->beep();
		}
	}

	return 1;
}

long textLabel::onEnter(FXObject* obj, FXSelector sel, void* ptr) {

	if (!_link.empty())
		setTextColor(FXRGB(255, 0, 0));

	return FXLabel::onEnter(obj, sel, ptr);
}

long textLabel::onLeave(FXObject* obj, FXSelector sel, void* ptr) {

	if (!_link.empty())
		setTextColor(FXRGB(0, 0, 255));

	return FXLabel::onLeave(obj, sel, ptr);
}

long textLabel::onTimer(FXObject*, FXSelector, void*) {

	getApp()->endWaitCursor();
	return 1;
}
