/*
 *  Copyright (C) 2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef TIMEDISPLAY_H
#define TIMEDISPLAY_H

#include <FX7Segment.h>

class timeDisplay : public FX7Segment {
	FXDECLARE(timeDisplay)

private:
	timeDisplay() {};
	timeDisplay(const timeDisplay&);
	timeDisplay& operator=(const timeDisplay&);

public:
	timeDisplay(FXComposite* p, FXObject* tgt=0, FXSelector sel=0) :
		FX7Segment(p, "--:--", SEVENSEGMENT_SHADOW|FRAME_NORMAL)
		{ setTarget(tgt); setSelector(sel); }
	virtual ~timeDisplay() {};

	void reset() { setText("--:--"); }
#if FOX_MINOR>6
	void setTime(const FXint seconds) { setText(FXString::value("%.2d:%.2d", seconds/60, seconds%60)); }
#else
	void setTime(const FXint seconds) { setText(FXStringFormat("%.2d:%.2d", seconds/60, seconds%60)); }
#endif
};

/*****************************************************************/

#include <FXLabel.h>

class timeLabel : public FXLabel {
	FXDECLARE(timeLabel)

private:
	timeLabel() {};
	timeLabel(const timeLabel&);
	timeLabel& operator=(const timeLabel&);

public:
	timeLabel(FXComposite* p) :
		FXLabel(p, "--:--", 0, FRAME_SUNKEN) {}
	virtual ~timeLabel() {};

	void reset() { setText("--:--"); }
#if FOX_MINOR>6
	void setTime(const FXint seconds) { setText(FXString::value("%.2d:%.2d", seconds/60, seconds%60)); }
#else
	void setTime(const FXint seconds) { setText(FXStringFormat("%.2d:%.2d", seconds/60, seconds%60)); }
#endif
};

#endif
