/*
 *  Copyright (C) 2009-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef TRACKLIST_H
#define TRACKLIST_H

#include <fx.h>

#include <new>
#include <FXArray.h>

/*****************************************************************/

class track_t {

private:
	FXString	_location;
	int		_start;

public:
	track_t() : _location(FXString::null), _start(0) {};
	track_t(const FXString& location, const unsigned int start=0) : _location(location), _start(start) {};

	const FXString& location() const { return _location; }

	const unsigned int start() const { return _start; }
};

/*****************************************************************/

class tracks : public FXArray<track_t> {

public:
	tracks(void) : FXArray<track_t>() {};

	tracks& append(const FXString &text) {
		const track_t t(text);
		return static_cast<tracks&>(FXArray<track_t>::append(t));
	}
};

/*****************************************************************/

class trackList {

public:
	virtual ~trackList() {};

	/// Load playlist
	virtual tracks* load() =0;

	/// Save playlist
	virtual bool save(const tracks*) =0;
};

#endif
