/*
 *  Copyright (C) 2009-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef TRACKLISTPLS_H
#define TRACKLISTPLS_H

#include "trackListBackend.h"

class trackListPls : public trackListBackend {

public:
	trackListPls(const FXString &path) : trackListBackend(path) {};

	/// Load playlist
	tracks* load() {
		FXSettings pls;
		pls.parseFile(_path, FALSE);

		const FXint items=pls.readIntEntry("playlist", "NumberOfEntries");

		for (int i=1; i<=items; i++) {
			const FXString file=pls.readStringEntry("playlist", FXStringFormat("File%d", i).text());
			_tracks->append(file);
		}

		return _tracks;
	}

	bool save(const tracks* tracks) {

		FXFile file(_path, FXIO::WriteOnly|FXIO::Create|FXIO::Truncate);

		writeLine(&file, "[playlist]");

		FXString tmp=FXStringFormat("NumberOfEntries=%d", tracks->no());
		writeLine(&file, tmp);

		for (int i=0; i<tracks->no(); i++) {
			tmp=FXStringFormat("File%d=%s", i+1, tracks->at(i).location().text());
			writeLine(&file, tmp);
		}

		file.close();
		return true;
	}
};

#endif
