/*
 *  Copyright (C) 2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef IO_H
#define IO_H

#ifdef _WIN32
#  include <windows.h>
#else
#  include <fcntl.h>
#  include <unistd.h>
#  include <errno.h>
#endif

#include <FXFile.h>

class FXFileEx : public FXFile {

private:
	FXFileEx(const FXFileEx&);
	FXFileEx &operator=(const FXFileEx&);

public:
	FXFileEx() : FXFile() {}

	bool lock(const FXlong start, const FXlong end) {
#ifdef _WIN32
		if (!LockFile(device, (DWORD)start, (DWORD)(start>>32), (DWORD)end, (DWORD)(end>>32)))
			return false;
#else
		struct flock fl;

		fl.l_type=F_WRLCK;
		fl.l_whence=SEEK_SET;
		fl.l_start=start;
		fl.l_len=end-start;

		if (fcntl(device, F_SETLK, &fl)==-1)
			return false;
#endif
		return true;
	}
};

/*****************************************************************/
/*
#include <FXPipe.h>

class FXPipeEx : public FXPipe {

private:
	FXPipeEx(const FXPipeEx&);
	FXPipeEx &operator=(const FXPipeEx&);

public:
	FXPipeEx() : FXPipe() {}
	FXPipeEx(FXInputHandle handle, FXuint  mode) : FXPipe(handle, mode) {}
	virtual ~FXPipeEx() {}

	static FXival readBlock(FXInputHandle handle, void* data, FXival count) {

		FXival nread=-1;
#ifdef WIN32
		//ConnectNamedPipe(handle, NULL);
		DWORD nr;
		if(::ReadFile(handle, data, (DWORD)count, &nr, NULL)!=0) {
			nread=(FXival)nr;
		}
		DisconnectNamedPipe(handle);
#else
		do {
			nread=::read(handle, data, count);
		} while (nread<0 && errno==EINTR);
#endif
		return nread;
	}
};
*/
#endif
